@@ -1,8 +1,8 @@
 from types import FunctionType
+from typing import Union
 
 import numpy as np
 import scipy.special as sps
-from plum import Union
 
 from . import B, dispatch, Numeric
 from ..custom import bvn_cdf as _bvn_cdf
