@@ -2,27 +2,26 @@
 from .. import *
 from .. import dispatch as dispatch_original
 from ..shape import dispatch_unwrap_dimensions
-from ..types import Number, NPNumeric, TFNumeric
+from ..types import NPNumeric, Number, TFNumeric
 
 dispatch = dispatch_unwrap_dimensions(dispatch_original)
 
-from plum import Union
+from typing import Union
 
 Numeric = Union[Number, NPNumeric, TFNumeric]
 
-from .generic import *
-from .shaping import *
-from .linear_algebra import *
-from .random import *
+from plum import clear_all_cache as _clear_all_cache
 
 import tensorflow as tf
 
 # noinspection PyUnresolvedReferences
-from ..types import _tf_retrievables
+from .generic import *
+from .linear_algebra import *
+from .random import *
+from .shaping import *
 
-# Retrieve types.
-for retrievable in _tf_retrievables:
-    retrievable.retrieve()
+# Clear cache to make sure that all newly loaded types are available.
+_clear_all_cache()
 
 # Set TF device manager.
 B.ActiveDevice._tf_manager = tf.device
